

############
#Objective
############

#This SURF function is 
#1) to select variables based on SURF algorithm
#2) to integrate all subfunctions into one R function

SURF<-function(Xo,y,X=NULL,fold=10,Alpha=1,prop=0.1,weights=FALSE,B=1000,C=200,ncores=1,display.progress=TRUE,family=stats::binomial(link="logit"),pval=0.05){

### X - count data, need to be converted to proportion
### Xo - standardised predictor variables
### y - response variables
### fold - number of folds for cross-validation in Lasso
### Alpha - alpha parameter for elastic net
### prop - proportion of observations left out in subsampling
### weights - use weighted regression for unbalanced class sizes
### B - number of subsamples to take
### C - number of permutations used to estimate null distribution
### in.parallel - whether SuRF should compute in parallel    
### display.progress - whether SuRF should print a message on completion of each
### task    

    if(family$link!=get(family$family)()$link){
        warning(paste("glmnet uses default link function ",get(family$family)()$link," whereas you have specified ",family$link,". Models used for ranking and variable selection won't match.",sep="\""))
    }
    pcutoff=1-pval

    if(!is.element("doParallel", utils::installed.packages()[,1])){
        ncores=1
    }

    

    traindata=dataclean(X.c=X,X.o=Xo,y=y)
    if(display.progress)  print("clean completed")



    tempmod=Subsample_B(B=B,data=traindata,fold=fold,Alpha=Alpha,prop=prop,weights=weights,ncores,family)
    if(display.progress)  print("subsample  completed")

    ranking=Ranking(data=traindata,model=tempmod)
    if(display.progress)    print("ranking completed")

    mod=seqcutoff(data=traindata,ranktable=ranking$table,C=C,weights=weights,ncores,family=family,pcutoff)
    if(display.progress)    print("select variable completed")


    ans<-list("Bmod"=tempmod,"trdata"=traindata,"ranklist"=ranking,"selmod"=mod,"family"=family)
    class(ans)<-"SURF_fit"
    return(ans)

}

###Print method for displaying output of SuRF

print.SURF_fit<-function(x,...){
    variables_selected<-x$selmod$vslist
    devs<-x$selmod$devlist
    pvals<-x$selmod$pvals
    table<-cbind("variable"=c(variables_selected,""),"deviance"=devs,"cut-off"=x$selmod$cutoff,"p-values"=pvals)
    row.names(table)<-NULL
    print(table)
}
