
############
#Objective
############
# This function is to run sub-sampling B times
# This is the version to be used if the cluster is available for parallel computing

#########
#INPUTS:
##########
#B: the number of sub-samplings to run (e.g., B=1000)


#data: the dataframe should be arranged in the way such that columns are
#X1,X2,X3....,Xp, status.  Where Xi's are variables and status is the binary outcome (0 or 1)

#fold: fold used in lasso cross validation to select the tuning parameter

#Type: should use 'class' for classification always

#Alpha: 1 for Lasso,0 for ridgeression

#prop: percentage of samples left out for each subsamping

#weights: 1 if weighted version is desired, 0 otherwise


##########
#OUTPUTS
##########

#Class.Err: misclassification error on the left out ones over B runs. A vector of length B.


#Lambda: tuning parameters selected from B runs. It is a vector of length B

#BETA: It is a matrix used to save the beta coefficients from all B runs. 
# Its dimension is B*(p+1). Each row represents the results from run i(i=1,2,3,...,B)
#Be aware of that the intercept is placed at the end for each run.



Subsample_B<-function(B,data,fold,Alpha,prop,weights,ncores,family){
    if(family$family=="binomial"){
###Should add option to use deviance for binomial distributions
        Type="class"
    }else{
        Type="deviance"
    }
  if(ncores>1){
      cl <- parallel::makeCluster(ncores) #not to overload your computer
      doParallel::registerDoParallel(cl)
  }

### User should register cluster prior to calling function

  
  var.list=NULL

#print(colnames(data))
  
  len=dim(data)[2]-1
  #length(which(substring(colnames(data),1,1)=="X"&(!is.na(as.numeric(substring(colnames(data),2))))))
  #This is the number of predictors
 
  
  if(ncores>1){
      RESULTS=foreach::`%dopar%`(foreach::foreach(i = seq_len(B), .combine = rbind), {
          model=Subsample.w(data=data,fold=fold,Alpha=Alpha,prop=prop,weights=weights,family,Type)
      
          beta=model$Beta
          lamb=model$lambda
                                        #var.list=c(var.list,model$coef[which(abs(as.numeric(model$coef[,2]))>eps),1][-1])
          error=model$Error
          out=c(error,lamb,beta)
          return(out)
          
      })
  }else{
      RESULTS=matrix(0,B,len+3)
      for(i in seq_len(B)){
          model=Subsample.w(data=data,fold=fold,Alpha=Alpha,prop=prop,weights=weights,family,Type)
          
          beta=model$Beta
          lamb=model$lambda
                                        #var.list=c(var.list,model$coef[which(abs(as.numeric(model$coef[,2]))>eps),1][-1])
          error=model$Error
          RESULTS[i,]=c(error,lamb,beta)          
      }     
  }
#  if(in.parallel){
#      on.exit(parallel::stopCluster(cl))
#  }
  return(list(Class.Err=sum(RESULTS[,1])/B,Lambda=RESULTS[,2],BETA=RESULTS[,-c(1:2)]))
}





