############
#Objective
############

#This function is to select variables sequentially
#it is made of two major steps:
#1)update deviance distributions until there is no variable selected based on the current thereshold (update.dev())
#2)select the variable based the current threshold(varselect())

##########
#INPUTS
##########
#data: data return from the object given by the dataclean() step
#ranktable: object return from the ranking step
#C: the number of permutation times

##########
#OUTPUTS
##########
#vslist: a list of variable being selected by SURF
#dist.mat: the dimension of this matrix depends on how many deviance distribution were created for selecting variables
         # each row contains the values of each distribution
         # There may be more rows than the number of selected variable indicating there was no variable selected by the last deviance distribution
# cutoff: cutoff values are 95% percentile of the distributions above. Again, there may be 1 more cutoff value than the number of selected 
         #indicating there was no variable selected by the last deviance distribution
#devlist: the deviance generated by each variable selected according to the order of the selected variables listed in vslist





###################################
#Other parameters in the function
###################################
#vclist: variable candiandiate list starting from the ranking list
#vslist: variable selected list
#vtlist: variable temporary list
#cutoff: a vector recording the cutoff value for each variable selected
#newdev.dist: a vector which forms a empirical distribution of deviance residuals
#dist.mat: a matrix whose row forms a empirical distribution of deviance residuals ( each value in "cutoff" is 95 percentile of this distribution)
#ranking: a list of variables that ranked by subsampling procedure(a vector of variable number, e.g., if the ranked list is X10,X5, X18, ranking=c(10,5,18))
#freq: frequency of above variables being selected by subsampling procedure


seqcutoff<-function(data,ranktable,C,weights,ncores,family,pcutoff){


  ranking=ranktable[,2]
  freq=ranktable[,1]


  vclist=ranking # vclist is a vector of ranked variables , each time, the
                  #selected variable will be sequentially removed from the this list
  cutoff=NULL # a vector of sequential cutoff values (possibly have 1 extra
        #values than #of selected variables if no variable is selected in the last round)
  vslist=NULL # a vector of selected variables
  devlist=NULL # a vector of deviance obtained by each selected variable
  pvallist=NULL # a vector of pvalues of selected variables
  dist.mat=NULL # a matrix with each row representing the emprical
                #distribution of deviance at each step

  len=length(vclist)



  # variables are slected sequentially. the new deviance disttribution will
     # be generated if
  repeat{
    size1=length(cutoff)
    size2=length(vslist)

    # print(i==1|((!(size1==0&&size2==0&&i>1))&&(size1==size2)))
    if(size1==size2) {
      newdev.dist=update.dev(data=data,vslist=vslist,C=C,weights=weights,ncores,family)
      cutoff=c(cutoff,stats::quantile(newdev.dist,prob=pcutoff))
      dist.mat=as.matrix(rbind(dist.mat,newdev.dist))

    }
#    print("updated dist")
#    print(cutoff[length(cutoff)])

temp=varselect(data=data,vclist=vclist,vslist=vslist,Cutoff=cutoff[length(cutoff)],freq=freq,weights=weights,family)
    devlist=c(devlist,temp$seldev)
    pval=sum(newdev.dist>temp$seldev)/length(newdev.dist)
    pvallist=c(pvallist,pval)
    vslist=temp$vslist
    vclist=temp$vclist
#    print(vslist)
    if(temp$tstat==1)break

  }


  return(list(vslist=vslist, dist.mat=dist.mat, cutoff=cutoff,
devlist=devlist,pvals=pvallist))
}

