############
#Objective
############

#This function is to select variables based on the latest derived deviance distribution
#This function is not to be used independently but will be called by the function seqcutoff()

##########
#INPUTS
##########
#data: the variable 'data' within seqcutoff()
#vslist: a vector of selected variables
#vclist: a vector of ranked variables to be selected
#Cutoff: cutoff deviance , 95% percentile of the current derived deviance distribution
#freq" frequecies for the variables appear in the current vclist based on the ranking process
##########
#OUTPUTS
##########
#vslist: updated vslist. any variable selected in current selection will be appended in this list. If none, will return the original list.
#vclist: updated vclist. any variable selected in current selection will be dropped in this list. If none, will return the original list.
#seldev: deviance given by the selected variable
#tstat: termination status   1: terminate the entire program   0: continue on variable selection


###################################
#Other notes for the function
###################################
#selection strategy idea is :
#a) start from the first variable in  the ranking table list (vclist)
#(latest one only, any variable being selected before, the variable will be dropped from this list, so this list only contain
# the ranking variables which are not selected yet)
#b) if the first variable is not selected, check the next variable and so on so forth....
#c) by anytime a variable is selected, the algorithm will drop this variable in the ranking table list and add this variable into 
# selected variable list (vslist). The 'tstat' will change from default 1 to 0 indicating a variable is selected succesfully.
# This will trigger next derivation of new deviance distribution. Variable selection process continues on....
# There is an exception that, if after selecting this new variable, the deviance residual becomes smaller than a threshold 1E-5
# then 'tstat' will change again from 0 to 1 to stopping deriving the new deviance distribution and the entire program will be ceased
#d) if there is no variable selected after screening through the entire canadidate list (vslist), this function will not change the status
# of 'tstat' and this is to be returned to seqcutoff() to stop deriving a new deviance distribution. Accordingly, the entire program 
# will be ceased. In this case, we will observe more deviance distributions than the number of selected variables.
#e)This particular version allows re-rank the variables which have ties. When the vclist contain variables with the same frequency,
   # we reorder these tied variables by comparing thier contribution in reducing the deviance residuals. 
   # The variable that reduces the most residual deviance will be ranked higher.





varselect<-function(data,vclist,vslist, Cutoff,freq,weights,family){

    len<-dim(data)[2]
    
  #calculating weights
    if(is.null(weights)==TRUE|weights==FALSE){
  #unweighted version
        w=rep(1,length(data[,len])) 
    }else {  #weighted version
        N1=length(which(data[,len]==1))
        N0=length(which(data[,len]==0))

        if(N1>=N0){
            w=ifelse(data[,len]==1,1,(N1/N0))
        }else{
            w=ifelse(data[,len]==1,(N0/N1),1)
        }
    }


  #tstat=1 indicating the current cutoff value select no variables from the
#current variable list vclist
  tstat=1
  seldev=NULL
  Len=length(vclist)
  tiemax=NULL
#  print(Len)

  for(i in seq_len(Len)){

    tie.ind= which(freq[i:Len]==freq[i])+i-1

    tielen=length(tie.ind)

    varmax=NULL
    for(k in seq_len(tielen)){
#      print( k)
#      print( vslist)
#      print(tie.ind)
#      print(vclist)
      vtlist=c(vslist,vclist[tie.ind[k]])
#      print( vtlist)
      dat=data.frame(data[,vtlist],status=data[,len])
      mod=stats::glm(status~.,data=dat,family=family,weights=w,control=list(maxit=500))
      ANOVA=stats::anova(mod,test="Chisq")
      vardev=ANOVA[length(vtlist)+1,2]

      varmax=c(varmax,vardev)

    }
    vtlist=c(vslist,vclist[tie.ind[which.max(varmax)]])
    tiemax=c(tiemax,max(varmax))
    vclist[tie.ind]=vclist[tie.ind][order(varmax,decreasing=T)]
    seldev=max(tiemax)
    if(tiemax[length(tiemax)]>=Cutoff){
      tstat=0
      vslist=vtlist
      vclist=vclist[-i]
      seldev=tiemax[length(tiemax)]
      if(ANOVA[length(vtlist)+1,4]<1E-5)tstat=1
      break
    }
#    print(i)
  }


  return(list(vslist=vslist, vclist=vclist,seldev=seldev,tstat=tstat))
}
